#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

from pydenji.config.pythonconfig import is_object_factory, is_eager

class UnconfiguredError(Exception):
    pass

class AppContext(object):
    def __init__(self, configuration):
        self._names_factories = self._get_all_factories(configuration)
        self._start(self._names_factories)

    def get_object(self, name, *args, **kwargs):
        try:
            return self._names_factories[name](*args, **kwargs)
        except KeyError:
            raise UnconfiguredError, "No factory was configured for %s" % name

    def _start(self, names_factories):
        for factory in names_factories.values():
            if is_eager(factory):
                factory()

    @staticmethod
    def _get_all_factories(config):
        # TODO: refactor using a list comprehension/filter expr.
        names_factories = {}
        for attr in dir(config):
            value = getattr(config, attr)
            if is_object_factory(value):
                names_factories[attr] = value
        return names_factories










#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni.

from functools import partial
from types import UnboundMethodType

_CONFIGURED_OBJECT_FACTORY = "_pydenji__CONFIGURED_OBJECT_FACTORY"
_INSTANTIATE_EAGERLY = "_pydenji__INSTANTIATE_EAGERLY"

def is_object_factory(obj):
    if getattr(obj, _CONFIGURED_OBJECT_FACTORY, None) is True:
        return True
    return False

def is_eager(obj):
    if getattr(obj, _INSTANTIATE_EAGERLY, None) is True:
        return True
    return False


class _Maybe(object):
    def __init__(self, has_value=False, value=None):
        self.has_value = has_value
        self.value = value

def singleton(func, eager=True):
    maybevalue = _Maybe()
    def singleton_wrapped(self, *args, **kwargs):
        if args or kwargs:
            raise TypeError, "Singleton mustn't take any parameter. Use per-instance config instead."

        if not maybevalue.has_value:
            maybevalue.value = func(self)
            maybevalue.has_value = True
        return maybevalue.value

    setattr(singleton_wrapped, _INSTANTIATE_EAGERLY, eager)
    setattr(singleton_wrapped, _CONFIGURED_OBJECT_FACTORY, True)

    return singleton_wrapped

singleton.lazy = partial(singleton, eager=False)

def prototype(func):
    f = partial(func)
    setattr(f, _CONFIGURED_OBJECT_FACTORY, True)
    setattr(f, _INSTANTIATE_EAGERLY, False)
    return f

def Configuration(cls, configure_with=singleton):
    """
    Makes all public, unwrapped methods *eager singletons* by default.
    Also, after instantiation a "params" instance attribute will be set -
    it will hold a dictionary.

    Non-public methods and already-wrapped methods will just go untouched.
    """
    configured_dict = {}
    for clsattr in dir(cls):
        attrvalue = getattr(cls, clsattr)
        if (not clsattr.startswith("_")) and isinstance(attrvalue, UnboundMethodType) and not is_object_factory(attrvalue):
            configured_dict[clsattr] = configure_with(attrvalue)
    return type(cls.__name__ + "Configuration", (cls, ), configured_dict)



from setuptools import setup, find_packages

setup(name='pydenji',
      version='0.1',
      description='Dependency Injection Toolkit',
      long_description="""
      Dependency Injection Toolkit.

      Enables easy dependency injection and declarative configuration for
      Python projects. While inspired by projects such as Spring Framework
      or Ruby's DIM, it is designed to be unobtrusive. It doesn't require
      modification in your business code to be employed; it can just be used in
      the outer layer, where configuration is actually needed.

      Small additional tools to help with configuration management are provided
      and can be used when and where are really fit.
      """,
      author='Alan Franzoni',
      license='APL 2.0',
      keywords='dependency injection inversion control ioc container',
      author_email='username@franzoni.eu',
      url='http://pydenji.franzoni.eu',
      packages=find_packages(exclude=["test"])
     )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (C) 2010 Alan Franzoni

from unittest import TestCase

from pydenji.appcontext import AppContext
from pydenji.config.pythonconfig import singleton
from pydenji.config.pythonconfig import Configuration

class TestAppContext(TestCase):
    def test_appcontext_allows_retrieving_by_name(self):
        @Configuration
        class MockConf(object):
            @singleton
            def something(self):
                return 1

        context = AppContext(MockConf())
        something = context.get_object("something")
        self.assertEquals(1, something)

    def test_appcontext_fetches_objects_eagerly_when_required(self):
        c = []
        @Configuration
        class MockConf(object):
            @singleton
            def something(self):
                c.append(True)
             

        conf = MockConf()
        context = AppContext(conf)
        self.assertEquals([True], c)

    def test_appcontext_fetches_objects_lazily_when_required(self):
        c = []
        @Configuration
        class MockConf(object):
            @singleton.lazy
            def something(self):
                c.append(True)


        conf = MockConf()
        context = AppContext(conf)
        self.assertEquals([], c)




